from os.path import dirname, join

DEBUG = True
TEMPLATE_DEBUG = DEBUG

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

DATABASES = {
    "default": {
        "ENGINE": 'django.db.backends.sqlite3',
        "NAME": join(dirname(__file__), 'db', 'friendlytagloader.db')
    }
}

INSTALLED_APPS = (
    'testproject',
    'friendlytagloader',
    'django.contrib.webdesign',
)

TEMPLATE_DIRS = (
    join(dirname(__file__), 'templates'),
)

ROOT_URLCONF = 'testproject.urls'

SECRET_KEY = 'AVx7Au7LXpdYTuqSUYaDGf4U'



from django.template import Library, TemplateSyntaxError
try:
    from django.template.base import (
        get_library, InvalidTemplateLibrary, TOKEN_BLOCK)
except ImportError:     # Django < 1.8
    from django.template import (
        get_library, InvalidTemplateLibrary, TOKEN_BLOCK)
from django.template.defaulttags import CommentNode, IfNode, LoadNode, load
from django.template.smartif import Literal

register = Library()


@register.tag
def friendly_load(parser, token):
    """
    Tries to load a custom template tag set. Non existing tag libraries
    are ignored.

    This means that, if used in conjunction with ``if_has_tag``, you can try to
    load the comments template tag library to enable comments even if the
    comments framework is not installed.

    For example::

        {% load friendly_loader %}
        {% friendly_load comments webdesign %}

        {% if_has_tag render_comment_list %}
            {% render_comment_list for obj %}
        {% else %}
            {% if_has_tag lorem %}
                {% lorem %}
            {% endif_has_tag %}
        {% endif_has_tag %}
    """
    bits = token.contents.split()
    if len(bits) >= 4 and bits[-2] == "from":
        try:
            load(parser, token)
        except TemplateSyntaxError:
            pass
    else:
        for taglib in bits[1:]:
            try:
                lib = get_library(taglib)
                parser.add_library(lib)
            except InvalidTemplateLibrary:
                pass
    return LoadNode()


def do_if_has_tag(parser, token, negate=False):
    """
    The logic for both ``{% if_has_tag %}`` and ``{% if not_has_tag %}``.

    Checks if all the given tags exist (or not exist if ``negate`` is ``True``)
    and then only parses the branch that will not error due to non-existing
    tags.

    This means that the following is essentially the same as a
    ``{% comment %}`` tag::

      {% if_has_tag non_existing_tag %}
          {% non_existing_tag %}
      {% endif_has_tag %}

    Another example is checking a built-in tag. This will always render the
    current year and never FAIL::

      {% if_has_tag now %}
          {% now "Y" %}
      {% else %}
          FAIL
      {% endif_has_tag %}
    """
    bits = list(token.split_contents())
    if len(bits) < 2:
        raise TemplateSyntaxError("%r takes at least one arguments" % bits[0])
    end_tag = 'end%s' % bits[0]
    has_tag = all([tag in parser.tags for tag in bits[1:]])
    has_tag = (not negate and has_tag) or (negate and not has_tag)
    nodelist_true = nodelist_false = CommentNode()
    if has_tag:
        nodelist_true = parser.parse(('else', end_tag))
        token = parser.next_token()
        if token.contents == 'else':
            parser.skip_past(end_tag)
    else:
        while parser.tokens:
            token = parser.next_token()
            if token.token_type == TOKEN_BLOCK and token.contents == end_tag:
                return IfNode([
                    (Literal(has_tag), nodelist_true),
                    (None, nodelist_false)
                ])
            elif token.token_type == TOKEN_BLOCK and token.contents == 'else':
                break
        nodelist_false = parser.parse((end_tag,))
        parser.next_token()
    return IfNode([(Literal(has_tag), nodelist_true),
                   (None, nodelist_false)])


@register.tag
def if_has_tag(parser, token):
    """
    Do something if all given tags are loaded::

       {% load friendly_loader %}
       {% friendly_load webdesign %}

       {% if_has_tag lorem %}
            {% lorem %}
       {% else %}
            Non dummy content goes here!
       {% endif_has_tag %}

    When given multiple arguments each and every tag in the list has to be
    available. This means that the following will render nothing::

       {% if_has_tag now nonexisting_tag %}
           {% now "Y" %}
       {% endif_has_tag %}
    """
    return do_if_has_tag(parser, token)


@register.tag
def ifnot_has_tag(parser, token):
    """
    Do something unless any given tag is loaded::

       {% load friendly_loader %}
       {% friendly_load comments %}

       {% ifnot_has_tag render_comment_list %}
            Comment support has been disabled.
       {% else %}
            {% render_comment_list for obj %}
       {% endifnot_has_tag %}

    In the case of multiple arguments, the condition will trigger if any tag in
    the list is unavailable. This means that the following will still render
    the current year::

       {% ifnot_has_tag now nonexisting_tag %}
           {% now "Y" %}
       {% endifnot_has_tag %}
    """
    return do_if_has_tag(parser, token, True)



from django.template import StringOrigin, Template, TemplateSyntaxError
try:
    from django.template.base import Lexer, Parser
except ImportError:     # Django < 1.8
    from django.template import Lexer, Parser
from django.template.context import Context
from django.test import TestCase


def _render_template(template):
    return Template(template).render(Context({})).strip()


class FriendlyLoadingTest(TestCase):

    def test_cannot_load_missing_taglib_using_standard_load(self):
        template = '{% load error_tags %}'
        self.assertRaises(TemplateSyntaxError, Template, template)

    def test_can_load_missing_taglib_using_friendly_load(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load error_tags %}')
        self.assertTrue(
            isinstance(Template(template), Template),
            'Expected template to initialize')

    def test_can_load_taglib_using_friendly_load(self):
        template = '{% load friendly_loader %}{% friendly_load webdesign %}'
        lexer = Lexer(template, StringOrigin(template))
        parser = Parser(lexer.tokenize())
        parser.parse()
        self.assertTrue(
            'lorem' in parser.tags,
            'Expected webdesign taglib to load and provide the lorem tag')

    def test_can_load_missing_and_existing_taglib_using_friendly_load(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load error_tags webdesign %}')
        lexer = Lexer(template, StringOrigin(template))
        parser = Parser(lexer.tokenize())
        parser.parse()
        self.assertTrue(
            'lorem' in parser.tags,
            'Expected webdesign taglib to load and provide the lorem tag')

    def test_can_load_from_taglib(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load lorem from webdesign %}')
        lexer = Lexer(template, StringOrigin(template))
        parser = Parser(lexer.tokenize())
        parser.parse()
        self.assertTrue(
            'lorem' in parser.tags,
            'Expected webdesign taglib to load and provide the lorem tag')

    def test_can_load_from_missing_taglib(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load error from error_tags %}')
        lexer = Lexer(template, StringOrigin(template))
        parser = Parser(lexer.tokenize())
        parser.parse()
        self.assertTrue(
            isinstance(Template(template), Template),
            'Expected template to initialize')


class BaseRenderTest(TestCase):
    def assertSuccess(self, template):
        self.assertEqual(
            'SUCCESS', _render_template(template),
            'Expected template to render SUCCESS')


class HasTagTest(BaseRenderTest):

    def test_must_have_arguments(self):
        template = (
            '{% load friendly_loader %}'
            '{% if_has_tag %}FAIL{% endif_has_tag %}')
        self.assertRaises(
            TemplateSyntaxError, _render_template, template)

    def test_can_test_builtins(self):
        template = (
            '{% load friendly_loader %}'
            '{% if_has_tag now %}SUCCESS{% endif_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_missing_tags(self):
        template = (
            '{% load friendly_loader %}'
            '{% if_has_tag fail %}{% fail %}'
            '{% else %}SUCCESS{% endif_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_loaded_tags(self):
        template = (
            '{% load friendly_loader webdesign %}'
            '{% if_has_tag lorem %}SUCCESS{% endif_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_friendly_loaded_tags(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load webdesign %}'
            '{% if_has_tag lorem %}SUCCESS{% endif_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_multiple_existing_tags(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load webdesign %}'
            '{% if_has_tag now lorem %}SUCCESS{% endif_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_both_existing_and_missing_tags(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load webdesign %}'
            '{% if_has_tag lorem fail %}FAIL'
            '{% else %}SUCCESS{% endif_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_missing_tags_without_else(self):
        template = (
            '{% load friendly_loader %}'
            '{% if_has_tag fail %}FAIL{% endif_has_tag %}')
        self.assertEqual(
            '', _render_template(template),
            'Expected template to render nothing')


class NotHasTagTest(BaseRenderTest):
    def test_must_have_arguments(self):
        template = (
            '{% load friendly_loader %}'
            '{% ifnot_has_tag %}FAIL{% endifnot_has_tag %}')
        self.assertRaises(TemplateSyntaxError, _render_template, template)

    def test_can_test_builtins(self):
        template = (
            '{% load friendly_loader %}'
            '{% ifnot_has_tag now %}FAIL'
            '{% else %}SUCCESS{% endifnot_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_missing_tags(self):
        template = (
            '{% load friendly_loader %}'
            '{% ifnot_has_tag fail %}SUCCESS'
            '{% else %}FAIL{% endifnot_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_loaded_tags(self):
        template = (
            '{% load friendly_loader webdesign %}'
            '{% ifnot_has_tag lorem %}FAIL'
            '{% else %}SUCCESS{% endifnot_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_friendly_loaded_tags(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load webdesign %}'
            '{% ifnot_has_tag lorem %}FAIL'
            '{% else %}SUCCESS{% endifnot_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_multiple_existing_tags(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load webdesign %}'
            '{% ifnot_has_tag now lorem %}FAIL'
            '{% else %}SUCCESS{% endifnot_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_both_existing_and_missing_tags(self):
        template = (
            '{% load friendly_loader %}'
            '{% friendly_load webdesign %}'
            '{% ifnot_has_tag lorem fail %}SUCCESS'
            '{% else %}FAIL{% endifnot_has_tag %}')
        self.assertSuccess(template)

    def test_can_test_tags_without_else(self):
        template = (
            '{% load friendly_loader %}'
            '{% ifnot_has_tag now %}FAIL{% endifnot_has_tag %}')
        self.assertEqual(
            '', _render_template(template),
            'Expected template to render nothing')



import codecs
from os.path import join, dirname
from setuptools import setup


version = '1.2.1'
read = lambda *names: codecs.open(join(dirname(__file__), *names),
                                  encoding='utf-8').read().strip()

setup(
    name='django-friendly-tag-loader',
    version=version,
    description='Want to optionally use a template tag library? Use this!',
    long_description='\n\n'.join((read('README'), read('CHANGES'),)),
    author='Jaap Roes',
    author_email='jaap.roes@gmail.com',
    url='https://bitbucket.org/jaap3/django-friendly-tag-loader/',
    package_dir={'': 'src'},
    packages=['friendlytagloader', 'friendlytagloader.templatetags'],
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    install_requires=[],
    zip_safe=False,
)
